#!/usr/bin/env node

var path = require('path')
var fse = require('fs-extra')
var watchman = require('fb-watchman');
var client = new watchman.Client();

const utils = require('./util')
const createStylefromCode = utils.createStylefromCode
const convertStylesToRNCSS = utils.convertStylesToRNCSS
var dir_of_interest = process.cwd()

client.capabilityCheck({optional:[], required:['relative_root']},
    function (error, resp) {
        if (error) {
            console.log(error);
            client.end();
            return;
        }

        // Initiate the watch
        client.command(['watch-project', dir_of_interest],
            function (error, resp) {
                if (error) {
                    console.error('Error initiating watch:', error);
                    return;
                }
                if ('warning' in resp) {
                    console.log('warning: ', resp.warning);
                }



                console.log('watch established on ', resp.watch,
                    ' relative_path', resp.relative_path);


                make_subscription(client, resp.watch, resp.relative_path)
            });
    });



function make_subscription(client, watch, relative_path) {
    var sub = {
        // Match any `.js` file in the dir_of_interest
        expression: ["allof", ["match", "*.wxss"]],
        // Which fields we're interested in
        fields: ["name", "size", "mtime_ms", "exists", "type"]
    };
    if (relative_path) {
        sub.relative_root = relative_path;
    }

    client.command(['subscribe', watch, 'mysubscription', sub],
        function (error, resp) {
            if (error) {
                // Probably an error in the subscription criteria
                console.error('failed to subscribe: ', error);
                return;
            }
            console.log('subscription ' + resp.subscribe + ' established');
        });

    // Subscription results are emitted via the subscription event.
    // Note that this emits for all subscriptions.  If you have
    // subscriptions with different `fields` you will need to check
    // the subscription name and handle the differing data accordingly.
    // `resp`  looks like this in practice:
    //
    // { root: '/private/tmp/foo',
    //   subscription: 'mysubscription',
    //   files: [ { name: 'node_modules/fb-watchman/index.js',
    //       size: 4768,
    //       exists: true,
    //       type: 'f' } ] }
    client.on('subscription', function (resp) {
        if (resp.subscription !== 'mysubscription') return;

        resp.files.forEach(function (file) {
            // convert Int64 instance to javascript integer
            const mtime_ms = +file.mtime_ms;



            console.log('file changed: ' + file.name, path.resolve(file.name), mtime_ms);
            const absPath = path.resolve(file.name)

            const cssStr = fse.readFileSync(absPath).toString()

            // const cssObj = convertStylesToRNCSS(createStylefromCode(cssStr.toString()))

            let cssObj = null

            if (cssStr.indexOf('@import') !== -1) {
                cssObj = "{}"
            } else {
                try {
                    cssObj = convertStylesToRNCSS(createStylefromCode(cssStr))
                } catch (e) {
                    cssObj = "{}"
                }
            }

            const newPath = path.resolve(dir_of_interest,
                'node_modules',
                'babel-plugin-import-wxss',
                'rncsscache',
                file.name + '.js'
            )
            const newDic = path.dirname(newPath)

            fse.mkdirsSync(newDic)
            fse.writeFileSync(newPath, `export default styles = ${cssObj}`)
        });
    });
}

