const fs = require('fs')
const transform = require('css-to-react-native').default

// import {Dimensions} from 'react-native'
const supportCss = new Set(["display", "width", "height", "start", "end", "left", "top", "right", "bottom", "minWidth", "maxWidth", "minHeight", "maxHeight", "margin", "marginVertical", "marginHorizontal",
    "marginBottom", "marginLeft", "marginRight", "marginTop", "marginStart", "marginEnd", "padding", "paddingBottom", "paddingHorizontal", "paddingLeft", "paddingRight", "paddingTop", "paddingVertical",
    "paddingStart", "paddingEnd", "borderWidth", "borderTopWidth", "borderStartWidth", "borderEndWidth", "borderBottomWidth", "borderRightWidth", "borderLeftWidth",
    "position", "flexDirection", "flexWrap", "justifyContent", "alignItems", "alignSelf", "alignContent", "overflow", "flex", "flexGrow", "flexShrink", "flexBasis",
    "aspectRatio", "zIndex", "direction", "shadowColor", "shadowOffset", "shadowOpacity", "shadowRadius", "transform", "transformMatrix", "decomposedMatrix", "scaleX", "scaleY",
    "rotation", "translateX", "translateY", "backfaceVisibility", "backgroundColor", "borderColor", "borderTopColor", "borderRightColor", "borderBottomColor", "borderLeftColor",
    "borderStartColor", "borderEndColor", "borderRadius", "borderBottomRightRadius", "borderBottomLeftRadius", "borderBottomStartRadius", "borderBottomEndRadius", "borderStyle",
    "opacity", "elevation", "color", "fontFamily", "fontSize", "fontStyle", "fontWeight", "fontVariant", "textShadowColor", "textShadowOffset", "textShadowRadius", "letterSpacing",
    "lineHeight", "textAlign", "textAlignVertical", "textDecorationColor", "textDecorationLine", "textDecorationStyle", "includeFontPadding", "writingDirection"])

function createStylefromCode(code) {
    code = code.replace(/\/\*[\s\S]*?\*\//g, "")

    function resolveObj(obj) {
        let out = {};
        for (let prop in obj) {
            if (supportCss.has(prop))
                out[prop] = obj[prop]
        }
        return out;
    }

    let clzMap = new Map();

    function _getName(str) {
        str = str.trim();
        return [str.substring(0, str.indexOf("{")).trim(), str.substring(str.indexOf('{')).trim()];
    }

    function _normalizeKey(k) {
        k = k.replace(/\s*>\s*/g, ">").replace(/\s*\+\s*/g, "+").replace(/\s*=\s*/g, "=").replace(/\s*~=\s*/g, "~=")
            .replace(/\s*\|=\s*/g, "|=")
        k = k.replace(/\s+/g, " ")
        return k
    }

    function _resolveVal(str) {
        str = str.replace(/{/g, '').replace(/}/g, '');
        let arr = str.split(";").map(e => e.trim()).filter(e => e.length > 0).map(e => e.split(":"));
        //check if multi value,choose last
        arr.forEach(e => {
            if (!!e[1] && e[1].indexOf(',') > 0) {
                e[1] = e[1].split(",").filter(e => !!e && e.trim().length > 0).pop();
            }
        })
        return transform(arr)
    }

    //正则，将rpx除以2
    code = code.replace(/\s*\n\s*/g, "").replace(/\d*\s*rpx/g, e => {
        return (e.substring(0, e.length - 3).trim()) / 2 + "px"
    }).split("}").filter(e => e.trim().length > 0).map(e => e + "}");

    for (let k = 0; k < code.length; k++) {
        let v = _getName(code[k]);
        let key = _normalizeKey(v[0])
        let val = _resolveVal(v[1])
        if (key.indexOf(",") >= 0) {
            key.split(",").filter(e => (!!e && e.trim().length > 0))
                .map(e => {
                    clzMap.set(e.trim(), Object.assign({}, val));
                })
        } else {
            clzMap.set(key, val)
        }
    }


    function formatOutput(resMap) {
        let out = {};
        for (let k of resMap.keys()) {
            out [k] = resolveObj(resMap.get(k))
        }
        return out;
    }

    return formatOutput(clzMap);
}

function createStyleDictionary(styles){
    let total = {}
    for (let k in styles) {
        const val = styles[k]
        k = sortSeq(k)
        const lastEle = getLastEle(k)
        if (!total.hasOwnProperty(lastEle)) {
            total[lastEle]=createItem(k, val)[lastEle]
        } else {
            merge(createItem(k, styles[k]), total)
        }
    }
    return total
}

function countCharFromStr(str, ch) {
    let strArr = [...str];
    let count = 0;
    for (let k in strArr) {
        if (strArr[k] === ch) {
            count++
        }
    }
    return count
}

const sortSeq = (str) => {
    return str.split(" ").filter(e => !!e && e.trim().length > 0).map(e => {
        if (countCharFromStr(e, '.') > 1) {
            return '.' + e.split(".").filter(e => !!e && e.trim().length > 0).sort().join(".")
        } else return e
    }).join(" ")
}

const createItem = (name, val) => {
    let nameArr = name.split(" ").filter(e => !!e && e.trim().length > 0).reverse()
    const map = {}
    let refMap = map
    let len = nameArr.length
    for (let ind in nameArr) {
        if (ind < len) {
            refMap[nameArr[ind]]={}
            refMap = refMap[nameArr[ind]]
        }
    }
    //结束标识
    refMap["_#_"]=val
    return map
}

//linka为单链，并入linkb
const merge = (linka, linkb) => {
    let refa = linka
    let refb = linkb
    //a为单链，所以每次遍历，key其实都只有一个,b可以有很多
    while (Object.keys(refa).length) {
        for (let key in refa) {
            if (refb.hasOwnProperty(key)) {
                if (key === '_#_') {
                    refb[key]=Object.assign({}, refa[key], refb[key])
                    return linkb
                }
                refa = refa[key]
                refb = refb[key]
                break
            } else {
                //找到不同，进行合并
                refb[key]=refa[key]
                return linkb
            }
        }
    }
}
const getLastEle = (str) => {
    return str.split(" ").filter(e => e.length > 0).pop()
}



function serailize(obj, str = "", whetherQuoto = true) {
    let tmp = '{';
    for (let k in obj) {
        if (obj.hasOwnProperty(k)) {
            if (whetherQuoto)
                tmp += "\"" + k + "\":"
            else tmp += k + ":"
            if (typeof obj[k] === 'object' && !Array.isArray(obj[k])) {
                if (k === '_#_')
                    tmp += serailize(obj[k], str, false)[0]
                else tmp += serailize(obj[k], str, true)[0]
            } else {
                if (whetherQuoto || isNaN(obj[k]) && !Array.isArray(obj[k]))
                    tmp += "\"" + obj[k] + "\"" + ","
                //may be array here
                else {
                    if (Array.isArray(obj[k])) {
                        tmp += serailizeArray(obj[k]) + ","
                    } else
                        tmp += obj[k] + ","
                }
            }
        }
    }
    tmp += "},"
    str = tmp + str
    return [tmp, str]
}

function serailizeArray(arr, str = "") {
    if (!Array.isArray(arr)) return serailize(arr)
    let ans = '['
    for (let k in arr) {
        if (Array.isArray(arr[k]))
            ans += serailizeArray(arr[k], str)
        else if (typeof arr[k] === 'object')
            ans += serailize(arr[k], "", false)[1].replace(/,}/g, '}')
        else if (typeof arr[k] === 'string')
            ans += arr[k]
    }
    ans = ans.substring(0, ans.length - 1)
    ans += ']'
    str = ans + str
    return str
}

function convertStylesToRNCSS(styles) {
    const cssMap = createStyleDictionary(styles)
    const str = serailize(cssMap)[1].replace(/,}/g, '}')
    return str.substring(0, str.length - 1)
}

module.exports = {
    createStylefromCode: createStylefromCode,
    convertStylesToRNCSS: convertStylesToRNCSS
}